function [li, PROBLEM, lik]=kalmanf(DD,ZZ,HH,CC,TT,RR,s00,P00,data)
% [LLH,FILTMV]=kalmanf(SYSM,s00,P00,data)
%
% calculates Kalman estimators using reduced form parameters
%
% REDUCED FORM
%   y(t) = DD + ZZ*s(t) + HH*eta(t)
%   s(t) = CC + TT*s(t-1) + RR*vep(t)
%   where
%       eta(t) ~ iid N(0,I)
%       vep(t) ~ iid N(0,I)
%       eta(t) and vep(t) are independent
%
% INPUT
%   SYSM : system matrices
%        .DD: ny by 1
%        .ZZ: ny by ns
%        .HH: ny by neta
%        .CC: ns by 1
%        .TT: ns by ns
%        .RR: ns by vep
%   s00  : initial mean, ns by 1
%   P00  : initial variance, ns by ns
%   DATA : data matrix, T by ny
%
% OUTPUT
%   LLH     : log likelihood, scalar
%   FILTMV  : filtered means and variances
%           .s0       : conditional mean z(t|t), t=0,...,T, ns by T+1
%           .s1       : mean of forecast z(t+1|t), t=0,...,T-1, ns by T
%           .ltrvecP0 : conditional variances of states,
%                       lower triangular part of P(t|t), ns*(ns+1)/2 by T+1
%           .ltrvecP1 : variances of forecasts of states,
%                       lower triangular part of P(t+1|t), ns*(ns+1)/2 by T

% Sungbae An
% Updated: 10/02/2004



% initialize
nobs = size(data,1);
ns   = size(CC,1);   % dimension of z
ny   = size(DD,1);      % dimension of y

%llik_c   = zeros(nobs,1);       % log likelihood contribution
%s0       = zeros(ns,nobs+1);    % E(s_t|t)
%s1       = zeros(ns,nobs);      % E(s_{t+1}|t)
%ltrvecP0 = zeros(ntr,nobs+1);   % var(s_t|t)
%ltrvecP1 = zeros(ntr,nobs);     % var(s_{t+1}|t)
NT   = 1;
s0t  = s00;
P0t  = P00;
li   = zeros(1,nobs);
% log likelihood by Kalman filter
for i=1:nobs

    % forecasting
    s1t = CC + TT*s0t; 
    P1t = TT*P0t*TT';
    P1t = P1t + RR*RR';
    P1t = symmat(P1t);

    % forecast error
    y1    = data(i,:)';
    e1    = y1-DD-ZZ*s1t;							% forecase error
    f1    = symmat(ZZ*P1t*ZZ'+HH*HH');
    if rcond(f1)<=1e-10 && i>NT;
        li = -inf;
        PROBLEM = 1;
        return;
    else
        f1inv = inv(f1);     % inverse of variance of forecast error
    end
    K     = P1t*ZZ'*f1inv;								% Kalman gain matrix

	% log likelihood
    li(i) = -0.5*ny*log(2*pi) + 0.5*log(det(f1inv)) - 0.5*e1'*f1inv*e1;
    % updating
    s0t = s1t + K*e1;
    temp = K*ZZ*P1t;
    P0t = P1t - temp;
end
lik = li;
li = sum(li(NT:end));
PROBLEM = 0;
if 1-isreal(li) || li>=0 || isnan(li);
        li = -inf;
        PROBLEM = 1;
        %save Theta_error Theta
        return;
end;
